@@ -1,14 +1,13 @@
 
-
 # Compute several weighted mean functions for a numeric vector
 
 .wmean.calculation <- function(data, w, mean.names) {
 
-         sapply(mean.names,
-                function(x, data, w) {
-                  get(paste("weighted_mean", x, sep = "_"))(data, w)
-                },
-                data = data, w = w)
+  sapply(mean.names,
+         function(x, data, w) {
+           get(paste("weighted_mean", x, sep = "_"))(data, w)
+         },
+         data = data, w = w)
 
 }
 
@@ -28,15 +27,15 @@
 
 .customCeiling <- function(x, Decimals = 1) {
 
-    x2 <- x * 10 ^ Decimals
-    ceiling(x2) / 10 ^ Decimals
+  x2 <- x * 10 ^ Decimals
+  ceiling(x2) / 10 ^ Decimals
 
 }
 
 .customFloor <- function(x, Decimals = 1) {
 
-    x2 <- x * 10 ^ Decimals
-    floor(x2) / 10 ^ Decimals
+  x2 <- x * 10 ^ Decimals
+  floor(x2) / 10 ^ Decimals
 
 }
 
@@ -94,10 +93,19 @@
       # .v <- (pi / 4) * tree[, "dbh"] ^ 2 * .column.height * 0.45
 
       # Paraboloid
-      .v <- pi * (.column.height ^ 2 / 2) * ((tree[, "dbh"] / 2) ^ 2 / (.column.height - 1.3) ^ 2)
+      .v <- pi * (.column.height ^ 2 / 2) * ((tree[, "dbh"] / 2) ^ 2 /
+                                               (.column.height - 1.3) ^ 2)
 
       tree <- cbind(tree, V.acum = cumsum(.v))
 
+      if(case == "field" & "w" %in% colnames(tree)){
+      # Biomass, accumulated from ntree = 1 to ntree = Ntree (m3/ha)
+      .w <- tree[, "w"]
+
+      tree <- cbind(tree, W.acum = cumsum(.w))
+
+      }
+
       # Mean diameters and heights
       if (!is.null(mean.names)) {
 
@@ -120,9 +128,42 @@
 
         # Mean heights (m)
         .mean.names <- mean.names[substr(names(mean.names),1, 1) == "h"]
-        .h <- .wmean(n = tree[, "n"], data = .column.height,
+
+        if(min(.column.height) < 1.3){
+
+          .n.ref <- data.frame(n = 1:length(.column.height),
+                               height = .column.height)
+          .n.ref <- .n.ref[which(.n.ref$height >= 1.3), ]
+
+          .column.height <- .column.height[which(.column.height >= 1.3)]
+          .n <- 1:length(.column.height)
+
+        # Mean heights (m)
+        .h <- .wmean(n = .n, data = .column.height,
                      mean.names = .mean.names)
-        tree <- cbind(tree, .h)
+        .h <- cbind(.n.ref, .h)
+        tree <- merge(tree, .h[, c("n", "h", "hg", "hgeom", "hharm")], by = "n", all.x = TRUE)
+
+        # Mean heights (m)
+        .h <- tree %>% tidyr::fill("h", .direction = "down")
+        tree$h <- .h$h
+
+        .hg <- tree %>% tidyr::fill("hg", .direction = "down")
+        tree$hg <- .hg$hg
+
+        .hgeom <- tree %>% tidyr::fill("hgeom", .direction = "down")
+        tree$hgeom <- .hgeom$hgeom
+
+        .hharm <- tree %>% tidyr::fill("hharm", .direction = "down")
+        tree$hharm <- .hharm$hharm
+
+        } else {
+
+          .h <- .wmean(n = tree[, "n"], data = .column.height,
+                       mean.names = .mean.names)
+          tree <- cbind(tree, .h)
+
+        }
 
       }
 
@@ -145,7 +186,7 @@
 
 
 # Compute a radius sequence, select trees according to argument 'radius.max',
-# compute accumulated number of points, and create a matrix containing the
+# compute accumulated number of points, and create a data.frame containing the
 # trees' data for each radius value - FIXED AREA PLOTS
 
 .radius.fixed.area.calculation <- function(radius.min, radius.increment,
@@ -181,16 +222,18 @@
   }
 
 
-  # Create a matrix containing the trees' data corresponding to each value in
-  # the previous radius sequence
+  # Create a data.frame containing the trees' data corresponding to each value
+  # in the previous radius sequence
   tree <-
     lapply(1:length(.radius.seq),
            function(n, x, y, data) {
              ind <- y == x[n]
              if (sum(ind) > 0) result <- data[ind, , drop = FALSE]
              else {
-               result <- matrix(NA, nrow = 1, ncol = ncol(data),
-                                dimnames = list(NULL, colnames(data)))
+               result <- data.frame(matrix(NA, nrow = 1, ncol = ncol(data),
+                                           dimnames = list(NULL,
+                                                           colnames(data))),
+                                    stringsAsFactors = FALSE)
                result[, "radius"] <- as.numeric(x[n])
              }
              return(result)
@@ -199,9 +242,7 @@
            y = .format.numb(x = tree[, "radius"], dec =num.dec), data = tree)
   tree <- do.call(rbind, tree)
 
-  # Fill missing values in the matrix using previous values
-
-  tree <- as.data.frame(tree, stringsAsFactors = FALSE)
+  # Fill missing values in the data.frame using previous values
 
   # Tree
   .tree <- tree %>% tidyr::fill("tree", .direction = "down")
@@ -227,6 +268,11 @@
     .V.acum <- tree %>% tidyr::fill("V.acum", .direction = "down")
     tree$V.acum <- .V.acum$V.acum
 
+    if(case == "field" & "w" %in% colnames(tree)){
+    # Biomass (Mg/ha)
+    .W.acum <- tree %>% tidyr::fill("W.acum", .direction = "down")
+    tree$W.acum <- .W.acum$W.acum}
+
     # Dbh (m)
     .dbh <- tree %>% tidyr::fill("dbh", .direction = "down")
     tree$dbh <- .dbh$dbh
@@ -272,15 +318,13 @@
       tree$num.points.est <- .num.points.est$num.points.est
 
       .num.points.hom.est <- tree %>% tidyr::fill("num.points.hom.est",
-                                           .direction = "down")
+                                                  .direction = "down")
       tree$num.points.hom.est <- .num.points.hom.est$num.points.hom.est
 
     }
 
   }
 
-  tree <- as.matrix(tree)
-
   return(tree)
 
 }
@@ -342,7 +386,7 @@
                           .col.names, drop = FALSE]
       colnames(distance.sampling) <- gsub("P", "EF", .col.names, fixed = TRUE)
       distance.sampling <- 10000 /
-                           (distance.sampling * pi * data[, "radius"] ^ 2)
+        (distance.sampling * pi * data[, "radius"] ^ 2)
       data <- cbind(data, distance.sampling)
 
     }
@@ -357,23 +401,25 @@
       # Compute shadows area
       .col.names <- c("horizontal.distance", "dbh", "partial.occlusion",
                       "wide")
-      .shadow <- cbind(data[data[, "radius"] <= .j, .col.names, drop = FALSE],
-                       shadow = NA)
+      .shadow <- as.matrix(cbind(data[data[, "radius"] <= .j, .col.names,
+                                      drop = FALSE],
+                                 shadow = NA))
 
-      .shadow[, "shadow"] <- ifelse(.shadow[, "partial.occlusion"] == 0,
+      .shadow[, "shadow"] <- ifelse(
+        .shadow[, "partial.occlusion"] == 0,
 
         # Non-occluded trees
         ((((pi * .j ^ 2) - (pi * .shadow[, "horizontal.distance"] ^ 2)) /
-          (2 * pi)) * atan2(.shadow[, "dbh"],
-                            .shadow[, "horizontal.distance"])) -
+            (2 * pi)) * atan2(.shadow[, "dbh"],
+                              .shadow[, "horizontal.distance"])) -
           ((pi * (.shadow[, "dbh"] / 2) ^ 2) / 2),
 
         # Occluded trees
         ((((pi * .j ^ 2) - (pi * .shadow[, "horizontal.distance"] ^ 2)) /
-          (2 * pi)) * (.shadow[, "wide"])) -
-        (((pi * (.shadow[, "dbh"] / 2) ^ 2) *
-          .shadow[, "wide"]) / atan2(.shadow[, "dbh"],
-                                     .shadow[, "horizontal.distance"])))
+            (2 * pi)) * (.shadow[, "wide"])) -
+          (((pi * (.shadow[, "dbh"] / 2) ^ 2) *
+              .shadow[, "wide"]) / atan2(.shadow[, "dbh"],
+                                         .shadow[, "horizontal.distance"])))
 
       # Correction: shadow = 0 if tree is not completely inside the plot
       .shadow[.shadow[, "horizontal.distance"] + .shadow[, "dbh"] / 2 > .j,
@@ -411,6 +457,24 @@
     .V <- data[, "V.acum"] * data[, .col.names, drop = FALSE]
     colnames(.V) <- gsub("EF", "V", colnames(.V), fixed = TRUE)
 
+    if(case == "field" & "W.acum" %in% colnames(data)){
+    # Biomass (Mg/ha)
+    .W <- data[, "W.acum"] * data[, .col.names, drop = FALSE]
+    colnames(.W) <- gsub("EF", "W", colnames(.W), fixed = TRUE)}
+
+    # Mean heights (m)
+    .h <- data %>% tidyr::fill("h", .direction = "down")
+    data$h <- .h$h
+
+    .hg <- data %>% tidyr::fill("hg", .direction = "down")
+    data$hg <- .hg$hg
+
+    .hgeom <- data %>% tidyr::fill("hgeom", .direction = "down")
+    data$hgeom <- .hgeom$hgeom
+
+    .hharm <- data %>% tidyr::fill("hharm", .direction = "down")
+    data$hharm <- .hharm$hharm
+
     # Mean diameters (cm), and mean heights (m)
     .col.names <- c("d", "dg", "dgeom", "dharm", "h", "hg", "hgeom", "hharm")
     .dh <- data[, .col.names, drop = FALSE]
@@ -440,7 +504,9 @@
 
   }
   .Plot <- cbind(.Plot, .G)
+  if (!is.null(case) & "w" %in% colnames(data)) .Plot <- cbind(.Plot, .V, .W, .dh)
   if (!is.null(case)) .Plot <- cbind(.Plot, .V, .dh)
+
   rownames(.Plot) <- NULL
 
   return(.Plot)
@@ -474,16 +540,20 @@
     .column.height <- switch(case, tls = data[, "P99"],
                              field = data[, "total.height"])
     .V <- cbind(V = pi * (data[, "dbh"] / 2) ^ 2 * .column.height * 0.45 *
-                    .N[, "N"])
+                  .N[, "N"])
+
+    if(case == "field" & "w" %in% colnames(data)){
+      # Biomass (Mg/ha)
+      .W <- cbind(W = data[, "w"] * .N[, "N"])}
 
     if (case == "tls") {
 
       # Correction of occlusion - Strahler et al. (2008)
       .De <- mean(data[, "dbh"]) *
-            (1 + (stats::sd(data[, "dbh"], na.rm = TRUE) /
-                  mean(data[, "dbh"], na.rm = TRUE)) ^ 2) ^ 0.5
+        (1 + (stats::sd(data[, "dbh"], na.rm = TRUE) /
+                mean(data[, "dbh"], na.rm = TRUE)) ^ 2) ^ 0.5
       .t <- ((.N[, "N"] / 10000) * .De * data[, "dbh"]) /
-            (2 * sqrt(BAF / 10000))
+        (2 * sqrt(BAF / 10000))
       .Ft <- (2 / .t ^ 2) * (1 - exp(-.t) * (1 + .t))
 
       # Density (trees / ha), basal area (m2 / ha) and volume (m3 / ha) with
@@ -508,9 +578,22 @@
 
       # Mean heights (m)
       .mean.names <- mean.names[substr(names(mean.names),1, 1) == "h"]
+
+      if(min(.column.height) < 1.3){
+
+        .column.height <- .column.height[which(.column.height >= 1.3)]
+
+        .h <- .wmean.calculation(data = .column.height,
+                                 w = rep(1, length(.column.height)),
+                                 mean.names = .mean.names)
+
+      } else {
+
       .h <- .wmean.calculation(data = .column.height, w = rep(1, nrow(data)),
                                mean.names = .mean.names)
 
+      }
+
       if (case == "tls") {
 
         names(.d) <- paste(names(.d), case, sep = ".")
@@ -526,7 +609,7 @@
   # Final vector with BAF, N (trees/ha), number of points, G (m2/ha),
   # V (m3/ha), mean diameters (cm), and/or mean heights (m)
   .Plot <- c()
-  if (is.null(case)) .Plot <- c(.Plot, data[1, "stratum"])
+  if (is.null(case)) .Plot <- c(.Plot, stratum = data[1, "stratum"])
   .Plot <- c(.Plot, BAF = BAF, apply(.N, 2, sum))
   if (!is.null(case) && case == "tls") {
 
@@ -536,11 +619,13 @@
   }
   .Plot <- c(.Plot, apply(.G, 2, sum))
   if (!is.null(case)) .Plot <- c(.Plot, apply(.V, 2, sum), .d, .h)
+  if (!is.null(case) & "w" %in% colnames(data)) .Plot <- c(.Plot, apply(.V, 2, sum), apply(.W, 2, sum), .d, .h)
+
   rownames(.Plot) <- NULL
 
   return(.Plot)
 
-  }
+}
 
 
 # Compute Pearson/Spearman correlations
@@ -663,3 +748,61 @@
 }
 
 
+# LiDAR metrics: "media","max","min","desv.std","varianza","moda","kurtosis","skewness", "por_sobre_moda", "por_sobre_media", "weibull_c", "weibull_b"
+
+.getmode <- function(v) {
+  uniqv <- unique(v)
+  uniqv[which.max(tabulate(match(v, uniqv)))]
+}
+
+.c_function<-function(c, media, varianza){
+  varianza-(media^2)*(gamma(1+2/c)-(gamma(1+1/c))^2)/(gamma(1+1/c))^2
+}
+
+.points.metrics <- function(rho_seq, data){
+
+  .metricas<-data.frame(matrix(nrow=0, ncol=13))
+
+  .data <- as.data.frame(data)
+
+  ####
+  .data <- .data[which(.data$z > 0.1), ]
+
+  for (radio in rho_seq) {
+
+    # .sub <- as.data.frame(data[data[, "rho"] <= radio,])
+    .sub <- .data[which(.data$rho <= radio), ]
+
+    .metricas_i<-data.frame(
+      # radio,
+      mean(.sub[, "z"]),
+      max(.sub[, "z"]),
+      min(.sub[, "z"]),
+      sd(.sub[, "z"]),
+      var(.sub[, "z"]),
+      .getmode(.sub[, "z"]),
+      moments::kurtosis(.sub[, "z"]),
+      moments::skewness(.sub[, "z"]))
+
+    names(.metricas_i)<-c("mean","max","min","sd","var","mode","kurtosis","skewness")
+
+    .cuenta_moda<-dim(.sub[which(.sub$z>.metricas_i$mode),])[1]
+    .cuenta_media<-dim(.sub[which(.sub$z>.metricas_i$mean),])[1]
+    .c<-uniroot(.c_function, media=.metricas_i$mean, varianza=.metricas_i$var, interval=c(min(.sub[, "z"]),max(.sub[, "z"])))$root
+    .b<-.metricas_i$mean/gamma(1+1/.c)
+    .metricas_i$perc_on_mean<-.cuenta_media*100/nrow(.sub)
+    .metricas_i$perc_on_mode<-.cuenta_moda*100/nrow(.sub)
+    .metricas_i$weibull_b<-.b
+    .metricas_i$weibull_c<-.c
+    .metricas<-rbind(.metricas,.metricas_i)
+
+  }
+
+  # .metricas <- .metricas[, 2:ncol(.metricas)]
+
+
+
+  return(.metricas)
+
+}
+
